From 5946ad35dafd109ed2218de421792d7a17e8a74d Mon Sep 17 00:00:00 2001
From: Hong Guan <hguan@ti.com>
Date: Wed, 3 Jul 2024 16:38:21 -0500
Subject: [PATCH 1/1] add reading SOC_UID in OPTEE via TISCI API

Signed-off-by: Hong Guan <hguan@ti.com>
---
 core/arch/arm/plat-k3/drivers/ti_sci.c          | 20 ++++++++++++++++++++
 core/arch/arm/plat-k3/drivers/ti_sci.h          |  2 ++
 core/arch/arm/plat-k3/drivers/ti_sci_protocol.h | 10 ++++++++++
 core/arch/arm/plat-k3/main.c                    |  7 +++++++
 4 files changed, 39 insertions(+)

diff --git a/core/arch/arm/plat-k3/drivers/ti_sci.c b/core/arch/arm/plat-k3/drivers/ti_sci.c
index 86001e7..03feec1 100644
--- a/core/arch/arm/plat-k3/drivers/ti_sci.c
+++ b/core/arch/arm/plat-k3/drivers/ti_sci.c
@@ -524,6 +524,26 @@ int ti_sci_set_keyrev(const uint8_t* dual_cert_ptr, uint32_t dual_cert_len)
 	return 0;
 }
 
+int ti_sci_get_soc_uid(uint32_t *soc_uid)
+{
+	struct ti_sci_msg_req_get_soc_uid req = { };
+	struct ti_sci_msg_resp_get_soc_uid resp = { };
+	struct ti_sci_xfer xfer = { };
+	int ret = 0;
+
+	ret = ti_sci_setup_xfer(TI_SCI_MSG_GET_SOC_UID, 0,
+				&req, sizeof(req), &resp, sizeof(resp), &xfer);
+	if (ret)
+		return ret;
+
+	ret = ti_sci_do_xfer(&xfer);
+	if (ret)
+		return ret;
+	memcpy(soc_uid, resp.soc_uid, sizeof(resp.soc_uid));
+	memzero_explicit(&resp, sizeof(resp));
+	return 0;
+}
+
 int ti_sci_init(void)
 {
 	struct ti_sci_msg_resp_version rev_info = { };
diff --git a/core/arch/arm/plat-k3/drivers/ti_sci.h b/core/arch/arm/plat-k3/drivers/ti_sci.h
index 2053192..b38fab8 100644
--- a/core/arch/arm/plat-k3/drivers/ti_sci.h
+++ b/core/arch/arm/plat-k3/drivers/ti_sci.h
@@ -184,6 +184,8 @@ int ti_sci_get_swrev(uint32_t *swrev);
 int ti_sci_get_keycnt_keyrev(uint32_t *key_cnt, uint32_t *key_rev);
 
 int ti_sci_set_keyrev(const uint8_t* dual_cert_ptr, uint32_t dual_cert_len);
+
+int ti_sci_get_soc_uid(uint32_t *soc_uid);
 /**
  * ti_sci_init() - Basic initialization
  *
diff --git a/core/arch/arm/plat-k3/drivers/ti_sci_protocol.h b/core/arch/arm/plat-k3/drivers/ti_sci_protocol.h
index 32868df..483e564 100644
--- a/core/arch/arm/plat-k3/drivers/ti_sci_protocol.h
+++ b/core/arch/arm/plat-k3/drivers/ti_sci_protocol.h
@@ -23,6 +23,7 @@
 #define TI_SCI_MSG_FWL_GET               0x9001
 #define TI_SCI_MSG_FWL_CHANGE_OWNER      0x9002
 #define TI_SCI_MSG_SA2UL_GET_DKEK        0x9029
+#define TI_SCI_MSG_GET_SOC_UID           0x9021
 #define TI_SCI_MSG_READ_OTP_MMR          0x9022
 #define TI_SCI_MSG_WRITE_OTP_ROW         0x9023
 #define TI_SCI_MSG_LOCK_OTP_ROW          0x9024
@@ -460,4 +461,13 @@ struct ti_sci_msq_req_set_keyrev {
 struct ti_sci_msq_resp_set_keyrev {
     struct ti_sci_msg_hdr hdr;
 } __packed;
+
+struct ti_sci_msg_req_get_soc_uid {
+	struct ti_sci_msg_hdr hdr;
+} __packed;
+
+struct ti_sci_msg_resp_get_soc_uid {
+	struct ti_sci_msg_hdr hdr;
+	uint32_t soc_uid[8];
+} __packed;
 #endif
diff --git a/core/arch/arm/plat-k3/main.c b/core/arch/arm/plat-k3/main.c
index 7f0382d..2f2bf62 100644
--- a/core/arch/arm/plat-k3/main.c
+++ b/core/arch/arm/plat-k3/main.c
@@ -72,6 +72,8 @@ static TEE_Result secure_boot_information(void)
 	uint32_t keycnt = 0;
 	uint32_t keyrev = 0;
 	uint32_t swrev = 0;
+	uint32_t soc_uid[8];
+	uint32_t i;
 	const uint8_t gKeyCert[KEYCERT_SIZE_IN_BYTES] __attribute__((aligned(128))) = KEYCERT;
 
 	if (!ti_sci_get_swrev(&swrev))
@@ -86,6 +88,11 @@ static TEE_Result secure_boot_information(void)
 	IMSG(">>>> ti_sci_set_keyrev() unit test start");
 	if (!ti_sci_set_keyrev(&gKeyCert[0], KEYCERT_SIZE_IN_BYTES))
 		IMSG(">>>> ti_sci_set_keyrev() return success!!!");
+	if (!ti_sci_get_soc_uid(soc_uid))
+		IMSG(">>>> SOC_UID READ success!!!");
+	for (i = 0; i < 8; i++){
+		IMSG("SOC_UID[%u] = 0x%08X\n", i, soc_uid[i]);
+	}
 
 	return TEE_SUCCESS;
 }
-- 
2.7.4

